"""
wsgid
~~~~~

A backgrounding front-end over cherrypy (builtin), twisted.web, circuits,
spawning, fapws3, and werkzeug, With server-independent support for code
reloading and pid file management.

Examples
========

Run a server in the foreground after importing `mywebapp.app` as the application
instance::

    wsgid --application mywebapp.app

Run a server in the foreground after importing and calling `mywebapp.create_app`
as the application factory::

    wsgid --application_factory mywebapp.create_app

Run a server using the pidfile `mypid.pid` after importing and
calling `mywebapp.create_app` as the application factory::

    wsgid --application_factory mywebapp.create_app --pidfile mypid.pid

Stop a server using the pidfile `mypid.pid`::

    wsgid --pidfile mypid.pid --stop

Use twisted.web's server to display builtin hello-world::

    wsgid --server=twistedweb

All these options have short versions, and can override defaults in config
files or the environment.


Using config files
==================

Ini-style config files can be used to provide any of the options available. For
example, myserver.ini::

    [ config ]

    pidfile = mypid.pid
    application_factory = mywebapp.crate_app

And then::

    wsgid -c myserver.ini

Is equivalent to the above examples. Note that the actual section titles in the
config file are ignored, and the file is essentially flattened.


Using the environment
=====================

Additionally any config variable can be overriden using the environment
variables. The variable name is uppercased, and prefixed with `WSGID_` to avoid
collisions with other apps for common names. For example::

    export WSGID_PIDFILE=mypid.pid

Is equivalent to passing the `--pidfile` on the command line.


Using the Werkzeug Debugger
===========================

Passing the option -d/--debug will wrap your WSGI application in the Werkzeug
debugger. It is not recommended to do this in production::

    wsgid --application mywebapp.app --foreground --debug


Using SSL
=========

The default CherryPy backend supports SSL, and this is enabled using the two
options --ssl_certificate/-C with --ssl_private_key/-K, To generate these as
an example::

    openssl genrsa 1024 > host.key
    openssl req -new -x509 -nodes -sha1 -days 365 -key host.key > host.cert

Then you can run your server like::

    wsgid --application mywebapp.app --foreground --ssl_certificate=host.cert --ssl_private_key=host.key




Complete options
================
Usage: wsgid [options]

Options:
  -h, --help            show this help message and exit
  -c CONFIG_FILE, --config_file=CONFIG_FILE
                        The configuration file
  -p PIDFILE, --pidfile=PIDFILE
                        The PID file
  -s, --stop            Stop the server.
  -a APPLICATION, --application=APPLICATION
                        The WSGI Applciation instance to import
  -A APPLICATION_FACTORY, --application_factory=APPLICATION_FACTORY
                        The WSGI Applciation factory to import
  -d, --debug           Run in the werkzeug debugger.
  -P PORT, --port=PORT  The port to listen on.
  -H HOST, --host=HOST  The host to listen on.
  -N, --no_reloader     Do not use the reloader.
  -L LOGDIR, --logdir=LOGDIR
                        The directory for logs.
  -w WORKDIR, --workdir=WORKDIR
                        The working directory for the daemon.
  -n SERVERNAME, --servername=SERVERNAME
                        The server name
  -C SSL_CERTIFICATE, --ssl_certificate=SSL_CERTIFICATE
                        The ssl certificate
  -K SSL_PRIVATE_KEY, --ssl_private_key=SSL_PRIVATE_KEY
                        The ssl private key
  -e VIRTUALENV, --virtualenv=VIRTUALENV
                        Path to a virtualenv to use
  -v, --verbose         Verbose logging
  -T, --no_log_stdout   Do not log on stdout
  -O SERVER, --server=SERVER
                        Server type to use, can be:
                            cherrypy (default)
                            twistedweb
                            circuitsweb
                            fapws3
                            spawningweb
                            wz
  -X, --dumb            Run a dumb server. Does not handle pidfiles/logs/etc.


Developer Information
=====================

Repository/tracker/wiki/etc at:

    http://bitbucket.org/aafshar/wsgid-main/

Latest tip package:

    http://bitbucket.org/aafshar/wsgid-main/get/tip.zip#egg=wsgid-dev
"""

version = '0.9'





"""
sensible config
~~~~~~~~~~~~~~~

So you have some configuration variables, and you want them to be available to
be in any number of ini-like files, as well as overridable from the environment,
and overridable from the command line. Define once, and use.

Our basic example::

    from config import Config, Option

    options = [
        Option('debug', 'Run in debug mode', False,
               short_name='d', converter=bool, action='store_true'),
    ]

    conf = Config(options)

    # Will start as the default value
    assert conf.debug == False
"""

import os, optparse, ConfigParser, pprint


class Option(object):

    def __init__(self, name, help, default, short_name=None, converter=unicode,
                       **optparse_kw):
        self.name = name
        if short_name is None:
            short_name = self.name[0]
        self.short_name = short_name
        self.help = help
        self.default = default
        self.converter = converter
        self.optparse_kw = optparse_kw

    @property
    def long_opt(self):
        return '--%s' % self.name

    @property
    def short_opt(self):
        return '-%s' % self.short_name


class ConfigFile(ConfigParser.ConfigParser):
    """Flat config parser"""

    def __init__(self, path):
        ConfigParser.ConfigParser.__init__(self)
        self.read(path)
        options = {}
        for section in self.sections():
            for name in self.options(section):
                options[name] = self.get(section, name)
        self.flat_options = options


class Config(object):

    def __init__(self, options, env_prefix=None):
        self._env_prefix = env_prefix
        self._parser = optparse.OptionParser()
        self._options = {}
        for opt in options:
            self.add_option(opt)

    def add_option(self, opt):
        self._options[opt.name] = opt
        self._parser.add_option(opt.short_opt, opt.long_opt,
                                help=opt.help, **opt.optparse_kw)
        setattr(self, opt.name, opt.default)

    def add_from_env(self, env=os.environ):
        for name, opt in self._options.items():
            if self._env_prefix:
                env_name = '%s_%s' % (self._env_prefix, name.upper())
            else:
                env_name = name.upper()
            raw = env.get(env_name)
            if raw:
                val = opt.converter(raw)
                setattr(self, name, val)

    def add_from_file(self, path):
        file = ConfigFile(path)
        for name, opt in file.flat_options.items():
            if name in self._options:
                val = self._options[name].converter(opt)
                setattr(self, name, val)

    def add_from_argv(self, argv):
        opts, args = self._parser.parse_args(argv)

        for name, opt in self._options.items():
            raw = getattr(opts, name, None)
            if raw:
                val = opt.converter(raw)
                setattr(self, name, val)

    def items(self):
        for name in self._options:
            yield name, getattr(self, name)

    def from_dict(self, d):
        for k, v in d.items():
            setattr(self, k, v)

    def to_dict(self):
        return dict(self.items())

    def pprint(self):
        pprint.pprint(self.to_dict())








from twisted.internet import reactor
from twisted.web import server
from twisted.web.wsgi import WSGIResource
from twisted.python.threadpool import ThreadPool
from twisted.internet import reactor

from wsgid.server import BaseWSGIServer

# Create and start a thread pool,
wsgiThreadPool = ThreadPool()
wsgiThreadPool.start()

# ensuring that it will be stopped when the reactor shuts down
reactor.addSystemEventTrigger('after', 'shutdown', wsgiThreadPool.stop)

# Create the WSGI resource

class WSGIServer(BaseWSGIServer):

    def start(self):
        self.log.info('Starting Twisted web server.')
        wsgi_res = WSGIResource(reactor, wsgiThreadPool, self.app)
        site = server.Site(wsgi_res)
        reactor.listenTCP(self.conf.port, site)
        reactor.run()

    def stop(self):
        self.log.debug('Stopping Twisted web server.')
        reactor.stop()






from werkzeug.serving import run_simple

from wsgid.server import BaseWSGIServer

class WSGIServer(BaseWSGIServer):
    name = 'Werkzeug'

    def start(self):
        run_simple(self.conf.host, self.conf.port, self.app, processes=1)







import os, sys, signal

import simplejson

from daemon import read_fd, open_pipes, create_logger
from utils import import_string
from config import Config
from main import options


def default_app_factory():
    def app(environ, start_response):
        start_response('200 OK', [('content-type', 'text/plain')])
        return ['hello', '\n', 'world']
    return app

default_app = default_app_factory()


class LoggedApp(object):

    def __init__(self, conf, app):
        self.conf = conf
        self.app = app
        self.log = create_logger(conf, 'access')

    def __call__(self, environ, start_response):
        self.log.info('%s %s' % (environ['SCRIPT_NAME'], environ['PATH_INFO']))
        return self.app(environ, start_response)

def logged_app(conf, oldapp):
    log = create_logger(conf, '')
    def app(environ, start_response, oldapp=oldapp, log=log):
        #log.info('%s %s' % (environ['SCRIPT_NAME'], environ['PATH_INFO']))
        return oldapp(environ, start_response)
    return app


def get_app(conf):
    if conf.application:
        app = import_string(conf.application)
    else:
        if conf.application_factory:
            app_factory = import_string(conf.application_factory)
            app = app_factory()
        else:
            app = default_app_factory()

    if conf.debug:
        app = debugged_app(app)

    #app = LoggedApp(conf, app)

    return app


def create_config(conf_data):
    conf = Config(options)
    conf.from_dict(conf_data)
    return conf

def debugged_app(app):
    from werkzeug.debug import DebuggedApplication
    return DebuggedApplication(app, True)

class BaseWSGIServer(object):

    name = 'unnamed'

    def __init__(self, conf, app):
        self.conf = conf
        self.app = app
        self.log = create_logger(conf, self.name)
        self.log.info('Starting %r server' % self.name)

    def start(self):
        pass

    def stop(self):
        pass



def activate_virtualenv(env):
    activate_script = os.path.join(env, 'bin', 'activate_this.py')
    execfile(activate_script, dict(__file__=activate_script))




class Server(object):

    def __init__(self, conf):
        self.conf = conf
        self.log = create_logger(conf, name='server')
        self.log.debug('Server process started.')
        if not (conf.application or conf.application_factory):
            self.log.warning('No application, or application_factory -> Hello World.')
        if conf.virtualenv:
            self.log.debug('Using virtualenv: %r' % conf.virtualenv)
            activate_virtualenv(conf.virtualenv)

        app = get_app(conf)
        self.server = self._create_server(app)

        if conf.ssl_certificate or conf.ssl_private_key:
            if conf.ssl_certificate and conf.ssl_private_key:
                self.server.ssl_certificate = conf.ssl_certificate
                self.server.ssl_private_key = conf.ssl_private_key
            else:
                log.warn('Must provide both ssl_certificate and ssl_private_key '
                         'for SSL support. Disabled.')


    def _on_sig15(self, signum, frame):
        self.log.debug('Server received SIG 15.')
        self.server.stop()

    def _pretty_display(self):
        opts = self.conf.to_dict()
        opts.update(dict(
            app = self.conf.application or self.conf.application_factory or 'Hello World',
            reloader = not self.conf.no_reloader,
            ve = self.conf.virtualenv or 'None',
            ssl = len(self.conf.ssl_certificate and self.conf.ssl_private_key) > 0,
            h = ((self.conf.host == u'0.0.0.0') and '*') or self.conf.host
        ))
        self.log.info(
            'Listening: %(h)s:%(port)s, '
            'app=%(app)r ssl=%(ssl)d, rld=%(reloader)d, dbg=%(debug)d, '
            'venv=%(ve)s' % opts)

    def start(self):
        self._pretty_display()
        signal.signal(15, self._on_sig15)
        try:
            self.server.start()
        except KeyboardInterrupt:
            self.log.debug('Server received KeyboardInterrupt.')
            self.server.stop()
        self.log.debug('Server ended.')
        self.server.stop()

    def _iter_module_filenames(self):
        """Get a list of all files in use"""
        for module in sys.modules.values():
            filename = getattr(module, '__file__', None)
            if filename:
                while not os.path.isfile(filename):
                    filename = os.path.dirname(filename)
                    if not filename:
                        break
                else:
                    if filename[-4:] in ('.pyc', '.pyo'):
                        filename = filename[:-1]
                    yield filename

    def _create_server(self, app):
        try:
            WSGIServer = import_string('wsgid.servers.%s:WSGIServer'
                                       % self.conf.server)
        except ImportError, e:
            self.log.warning('Chosen server %r not available: %s' %
                             (self.conf.server, e))
            WSGIServer = import_string('wsgid.servers.cherrypy:WSGIServer')
        server = WSGIServer(self.conf, app)
        return server


class ServerProcess(Server):
    def __init__(self, inpath, outpath):
        fin, fout = open_pipes(inpath, outpath)
        data = read_fd(fin)
        conf_data = simplejson.loads(data)
        conf = create_config(conf_data)
        Server.__init__(self, conf)
        module_filenames = simplejson.dumps(list(self._iter_module_filenames()))
        os.write(fout, module_filenames)
        os.close(fin)
        os.close(fout)



def run_process_server(inpath, outpath):
    p = ServerProcess(inpath, outpath)
    p.start()


if __name__ == '__main__':
    if '--reload-loop' in sys.argv:
        run_process_server(*sys.argv[sys.argv.index('--reload-loop') + 1:])






from wsgid.server import BaseWSGIServer

import fapws._evwsgi as evwsgi
from fapws import base

class WSGIServer(BaseWSGIServer):

    name = 'fapws3'

    def start(self):
        evwsgi.start(self.conf.host, self.conf.port)
        evwsgi.set_base_module(base)
        evwsgi.wsgi_cb(('', self.app))
        self.log.info('Starting Fapws3 web server')
        #evwsgi.set_debug(0)
        evwsgi.run()

    def stop(self):
        pass







import os, sys, subprocess

from wsgid.server import BaseWSGIServer, get_app, create_config

from spawning.spawning_controller import run_controller, DEFAULTS

class ConfigHolder(object):
    """A config holder"""
    app = None


class WSGIServer(BaseWSGIServer):

    def start(self):
        if not self.conf.application:
            self.log.warn('For spawning, must provide an importable app instance')
            self.conf.application = 'wsgid.server.default_app'
        factory_args = DEFAULTS.copy()
        factory_args.update({
            'verbose': False,
            'host': self.conf.host,
            'port': self.conf.port,
            'access_log_file': os.path.join(self.conf.logdir, 'access.log'),
            'coverage': False,
            'args': [self.conf.application],
            'wsgid_conf': self.conf.to_dict(),
            'num_processes': 4,
        })

        run_controller('wsgid.servers.spawningweb.config_factory',
                       factory_args)


def config_factory(args):
    args['app_factory'] = 'wsgid.servers.spawningweb.app_factory'
    return args


def app_factory(args):
    conf = create_config(args.get('wsgid_conf', {}))
    app = get_app(conf)
    return app







from wsgid.server import BaseWSGIServer


from circuits.web import BaseServer, wsgi
from circuits.core.workers import cpus, processes
from circuits.net.pollers import EPoll, Poll, Select


class WSGIServer(BaseWSGIServer):
    name = 'circuitsweb'

    def start(self):
        self.server = BaseServer(self.conf.port, poller=Select)
        self.server += wsgi.Gateway(self.app)
        for i in xrange(cpus() - 1):
            self.server.start(process=True)
        self.server.run()

    def stop(self):
        self.server.stop()
        for p in processes():
            p.terminate()







from wsgid.server import BaseWSGIServer
from cherryserver import CherryPyWSGIServer


class WSGIServer(CherryPyWSGIServer, BaseWSGIServer):

    name = 'cherrypy'

    def __init__(self, conf, app):
        BaseWSGIServer.__init__(self, conf, app)
        CherryPyWSGIServer.__init__(self,
            (conf.host, conf.port),
            app,
            server_name = conf.servername
        )

    def start(self):
        CherryPyWSGIServer.start(self)






from glashammer.application import make_app
from glashammer.utils import Response

from werkzeug import Response, Request


def do_root(req):
    return Response('hello world')

def do_except(req):
    raise Exception

@Request.application
def app(req):
    print req.path
    if req.path == '/':
        return do_root(req)
    elif req.path == '/except':
        return do_except(req)










"""
wsgid.daemon
~~~~~~~~~~~~
"""

import sys, os, time, atexit, signal, errno, tempfile, subprocess
import logging
from logging import handlers

import simplejson


class DaemonAlreadyRunningError(RuntimeError):
    """The daemon is already running"""


class DaemonNotRunningError(RuntimeError):
    """The daemon is not running"""


def create_dummy_pidfile():
    fd, filename = tempfile.mkstemp(prefix='wsgid_', suffix='.pid')
    os.close(fd)
    return filename


def create_pidfile(pidfile):
    pid = os.getpid()
    f = open(pidfile, 'w')
    f.write('%s\n' % pid)
    f.close()


def remove_pidfile(pidfile):
    try:
        os.unlink(pidfile)
    except OSError:
        pass


def remove_pidfile_factory(pidfile):
    def _remove(pidfile=pidfile):
        remove_pidfile(pidfile)
    return _remove


def get_pid(pidfile):
    try:
        f = open(pidfile)
        pd = f.read()
        f.close()
        try:
            pid = int(pd)
        except ValueError:
            pid = None
    except IOError:
        pid = None
    return pid


def is_pid_running(pid):
    try:
        os.kill(pid, 0)
        running = True
    except OSError, err:
        running = (err.errno == errno.EPERM)
    return running


def is_daemon_running(pidfile):
    pid = get_pid(pidfile)
    if pid:
        if is_pid_running(pid):
            running = True
        else:
            remove_pidfile(pidfile)
            running = False
    else:
        running = False
    return running


def signal_daemon(pidfile, signum):
    if is_daemon_running(pidfile):
        pid = get_pid(pidfile)
        os.kill(pid, signum)
    else:
        raise DaemonNotRunningError(
            'No running daemon at pidfile %r.' % pidfile)


def stop_daemon(pidfile):
    signal_daemon(pidfile, 15)


log_format = ('%(asctime)s:%(levelname)8s:%(name)12s:%(process)d:'
              '%(message)s')


def create_logger(conf, name='daemon', filename='wsgid.log'):
    if not os.path.exists(conf.logdir):
        os.mkdir(conf.logdir)
    if conf.verbose:
        conf.loglevel = logging.DEBUG
    else:
        conf.loglevel = logging.INFO
    handler = handlers.RotatingFileHandler(
              os.path.join(conf.logdir, filename),
              maxBytes=200000, backupCount=20)
    formatter = logging.Formatter(log_format, '%d.%m.%Y:%H:%M:%S')
    handler.setFormatter(formatter)
    log = logging.getLogger(name)
    log.addHandler(handler)
    log.setLevel(conf.loglevel)

    if not conf.no_log_stdout:
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        log.addHandler(handler)
    return log


def read_fd(fd, chunksize=1024):
    buffer = []
    running = True
    while running:
        chunk = os.read(fd, chunksize)
        buffer.append(chunk)
        running = len(chunk) == chunksize
    return ''.join(buffer)


class WSGIRunner(object):

    def __init__(self, config):
        self.config = config

    def start(self):
        from server import Server
        self.server = Server(self.config)
        self.server.start()
        

class WSGIDaemon(object):

    def __init__(self, config):
        self.config = config
        self.stopped = False
        if not self.config.pidfile:
            self.config.pidfile = create_dummy_pidfile()
        self.pidfile = config.pidfile
        self.log = create_logger(config)
        self.log.debug('Initialized daemon.')
        for k, v in config.items():
            if v:
                self.log.debug('Config: %s=%r' % (k, v))
        self.log.debug('Listening on http://%s:%s.' % (config.host, config.port))

    def _on_signal_15(self, signum, frame):
        self.log.debug('Received signal 15. Will stop.')
        self.stop()
        self.log.info('Stopped.')

    def _on_exit(self):
        self.stop()

    def start(self):
        if is_daemon_running(self.pidfile):
            self.log.error('Daemon already running.')
            return 1
        else:
            self.log.debug('Starting daemon.')
            create_pidfile(self.config.pidfile)
            self.log.debug('Created PID File "%s".' % self.pidfile)
            signal.signal(15, self._on_signal_15)
            atexit.register(self._on_exit)
            self.log.debug('Registered signal handlers.')
            server_config = self.config
            self.child_pid = None
            self.child = None
            self.running = True
            if server_config.no_reloader:
                self.log.debug('Not using reloader.')
            else:
                self.log.debug('Using reloader.')
            self._wait_as_reloader()
            self.log.info('Daemon finished naturally.')
            return 0

    def stop(self):
        if not self.stopped:
            self.running = False
            self._stop_child()
            remove_pidfile(self.pidfile)
            self.log.debug('Exited. Removing PID file "%s".' % self.pidfile)
            self.stopped = True

    def _stop_child(self):
        self.log.debug('Stopping server.')
        if self.child_pid is not None:
            os.kill(self.child_pid, 15)
            time.sleep(1)
            try:
                os.kill(self.child_pid, 9)
            except OSError:
                pass
        if self.child is not None:
            try:
                self.child.communicate()
            except KeyboardInterrupt:
                self.log.warn('Ok, if you say so. Killing')
                os.kill(self.child.pid, 9)
            returncode = self.child.returncode
            self._child_waited()
            return returncode

    def _child_waited(self):
        self.log.debug('Server exited with code %s. '
                        % self.child.returncode)
        self.child = None
        self.child_pid = None

    def _wait_as_reloader(self):
        while self.running:
            if not self.child:
                p = self._fork_server(self.config)
            if not p:
                self.running = False
                continue
            try:
                changed_filename = self._reloader_loop()
            except KeyboardInterrupt:
                print # to make it pretty on stdout
                self.log.info('Received Keyboard Interrupt.')
                self.running = False
                continue
            if not self.config.no_reloader and self.running:
                self.log.info('Reloading for change to %r.' % changed_filename)
                self._stop_child()

    def _fork_server(self, server_config):
        self.log.debug('Spawning server.')
        inpath, outpath = create_pipe_names()

        runner = os.path.join(os.path.dirname(__file__), 'server.py')

        p = subprocess.Popen([sys.executable, runner, '--reload-loop', outpath, inpath],
            stdout=sys.stdout,
            stderr=sys.stderr,
            stdin=subprocess.PIPE,
            env=os.environ.copy())

        create_pipes(inpath, outpath)
        fin, fout = open_pipes(inpath, outpath)

        conf_dump = simplejson.dumps(server_config.to_dict())
        os.write(fout, conf_dump)

        d = read_fd(fin)
        try:
            modules_list = simplejson.loads(d)
        except Exception, e:
            self.log.error('Could not start server.')
            retcode = p.wait()
            self._child_waited(retcode)
            return
        self.modules_list = modules_list
        self.child_pid = p.pid
        self.child = p

        os.close(fin)
        os.close(fout)
        os.unlink(inpath)
        os.unlink(outpath)
        return p

    def _reloader_loop(self, extra_files=None, interval=1):
        mtimes = {}
        while self.running:
            for filename in self.modules_list:
                try:
                    mtime = os.stat(filename).st_mtime
                except OSError:
                    continue

                old_time = mtimes.get(filename)
                if old_time is None:
                    mtimes[filename] = mtime
                    continue
                elif mtime > old_time:
                    return filename
            time.sleep(interval)



def create_temp_path():
    fd, path = tempfile.mkstemp()
    os.close(fd)
    os.unlink(path)
    return path

def create_pipe_names():
    return create_temp_path(), create_temp_path()

def create_pipes(inpath, outpath):
    os.mkfifo(inpath)
    os.mkfifo(outpath)

def open_pipes(inpath, outpath):
    flags = os.O_RDWR | os.O_SYNC
    f1 = os.open(inpath, flags)
    f2 = os.open(outpath, flags)
    return f1, f2







"""
Main runner.
"""

import os, sys

from daemon import DaemonAlreadyRunningError, DaemonNotRunningError, \
    WSGIDaemon, stop_daemon, create_logger, WSGIRunner
from config import Config, Option


def stop(conf):
    log = create_logger(conf, 'stopper')
    if not conf.pidfile:
        log.error('Need a pidfile to stop.')
        return 1
    else:
        try:
            stop_daemon(conf.pidfile)
            log.info('Stopped %r' % conf.pidfile)
            return 0
        except DaemonNotRunningError, e:
            log.error('No running daemon for %r' % conf.pidfile)
            return 1


def start(conf):
    if conf.dumb:
        daemon = WSGIRunner(conf)
    else:
        daemon = WSGIDaemon(conf)
    daemon.start()

servers = 'cherrypy (default), twistedweb, circuitsweb, fapws3, spawningweb, wz'

options = [
    Option('config_file', 'The configuration file', '', 'c'),
    Option('pidfile', 'The PID file', '', 'p'),
    Option('stop', 'Stop the server.', False, 's', bool, action='store_true'),
    Option('application', 'The WSGI Applciation instance to import', '', 'a'),
    Option('application_factory', 'The WSGI Applciation factory to import',  '', 'A'),
    Option('debug', 'Run in the werkzeug debugger.', False, 'd', bool, action='store_true'),
    Option('port', 'The port to listen on.', 9090, 'P', int),
    Option('host', 'The host to listen on.', '0.0.0.0', 'H'),
    Option('no_reloader', 'Do not use the reloader.', False, 'N', bool, action='store_true'),
    Option('logdir', 'The directory for logs.', os.path.join(os.getcwd(), 'logs'), 'L'),
    Option('workdir', 'The working directory for the daemon.', os.getcwd(), 'w'),
    Option('servername', 'The server name', '', 'n'),
    Option('ssl_certificate', 'The ssl certificate', '', 'C'),
    Option('ssl_private_key', 'The ssl private key', '', 'K'),
    Option('virtualenv', 'Path to a virtualenv to use', '', 'e'),
    Option('verbose', 'Verbose logging', False,
           'v', bool, action='store_true'),
    Option('no_log_stdout', 'Do not log on stdout', False, 'T', bool,
            action='store_true'),
    Option('server', 'Server type to use. One of: %s' % servers, 'cherrypy', 'O'),
    Option('dumb', 'Run a dumb server. Does not handle pidfiles/logs/etc.', False, 'X', bool, action='store_true'),
]


def get_config(argv):
    # we need to do this twice, so we can have the config file
    conf = Config(options, 'WSGID')
    conf.add_from_env()
    conf.add_from_argv(argv)
    config_file = conf.config_file
    if config_file:
        conf = Config(options, 'WSGID')
        conf.add_from_file(config_file)
        conf.add_from_env()
        conf.add_from_argv(argv)
    return conf


def main(argv):
    conf = get_config(argv)
    if conf.stop:
        return stop(conf)
    else:
        return start(conf)


if __name__ == '__main__':
    sys.exit(main(sys.argv))





import os, sys

import subprocess, time

all = [
    #'cherrypy',
    #'fapws3',
    #'spawningweb',
    'twistedweb',
    #'circuitsweb',
    #'wz',
]

try:
    os.mkdir('bench_out')
except OSError:
    pass

processes = []

results = []

sys.stderr.write('performing benchmark for %s\n' % all)
for rps in ['1', '100', '1000']:
    sys.stderr.write('c=%4s' % rps)
    for sname in all:
        sys.stderr.write(' %s:' % sname)
        daemon = subprocess.Popen(
            ['wsgid', '-T', '-O', sname, '-p', 'benchmark.pid', '-N'],
            stdout=open(os.path.join('bench_out', 'benchout.txt'), 'a'),
            stderr=open(os.path.join('bench_out', 'benchout.txt'), 'a'),
        )
        processes.append(daemon.pid)
        time.sleep(1)
        tester = subprocess.Popen(
            ['ab', '-n', '1000', '-c', rps,
             'http://localhost:9090/'],
             stdout=subprocess.PIPE,
             stderr=open(os.path.join('bench_out', 'benchout.txt'), 'a')
        )

        output, a = tester.communicate()

        stopper = subprocess.Popen(['wsgid', '-s', sname, '-p', 'benchmark.pid'],
            stdout=open(os.path.join('bench_out', 'benchout.txt'), 'a'),
            stderr=open(os.path.join('bench_out', 'benchout.txt'), 'a'),
        )
        stopper.communicate()
        daemon.communicate()

        sys.stderr.write('(%s,%s,%s)' % (
            daemon.returncode,
            tester.returncode,
            stopper.returncode
        ))


        pid = daemon.pid
        try:
            os.kill(pid, 15)
        except OSError:
            pass
        try:
            os.kill(pid, 9)
        except OSError:
            pass

        for line in output.splitlines():
            if 'Requests per second' in line:
                results.append((
                    float(line.strip().split(':', 1)[-1].strip().split()[0]),
                    sname,
                    rps
                ))
    sys.stderr.write('\n')
    sys.stderr.flush()

sys.stderr.write('Finished. Results:\n')
sys.stderr.flush()

for pid in processes:
    try:
        os.kill(pid, 15)
        time.sleep(1)
    except OSError:
        pass
    try:
        os.kill(pid, 9)
    except OSError:
        pass


print '%8s\t%18s\t%8s' % ('--------', '------------------', '--------')
print '%8s\t%18s\t%8s' % ('Req/s', 'Server', 'c')
print '%8s\t%18s\t%8s' % ('--------', '------------------', '--------')
for res in reversed(sorted(results)):
    print '%8s\t%18s\t%8s' % res








# From Werkzeug
def import_string(import_name, silent=False):
    """Imports an object based on a string.  This is useful if you want to
    use import paths as endpoints or something similar.  An import path can
    be specified either in dotted notation (``xml.sax.saxutils.escape``)
    or with a colon as object delimiter (``xml.sax.saxutils:escape``).

    If `silent` is True the return value will be `None` if the import fails.

    :param import_name: the dotted name for the object to import.
    :param silent: if set to `True` import errors are ignored and
                   `None` is returned instead.
    :return: imported object
    """
    import_name = str(import_name)
    try:
        if ':' in import_name:
            module, obj = import_name.split(':', 1)
        elif '.' in import_name:
            items = import_name.split('.')
            module = '.'.join(items[:-1])
            obj = items[-1]
        else:
            return __import__(import_name)
        return getattr(__import__(module, None, None, [obj]), obj)
    except (ImportError, AttributeError):
        if not silent:
            raise


